@@ -63,6 +63,9 @@ __authors__  = (
     'Ariset Llerena',
     'Adam Malcontenti-Wilson',
     'Tobias Bell',
+    'Naglis Jonaitis',
+    'Charles Chen',
+    'Hassaan Ali',
 )
 
 __license__ = 'Public Domain'
